# frozen_string_literal: true

# Legende
# Les script a plusieurs parties indique que les bash (Batch dans l'app)
# peuvent s'initier avec differents types de source
# Le besoin est donc d'isoler la logique associee au modele actuel
# ne pas oublier d'indexer les champs simples

# TODO: Réflechir au set de housing_requests_ids (validate main_applicant)
#
#
module Hermes
  # Applicant class
  class Applicant
    include Hermes::Concerns::CommonFieldsConcern
    include Hermes::Concerns::Status
    include Hermes::Concerns::ChangeMinusTwoYearConcern
    include ::Concerns::NotificationCenter::Targetable
    include ::Concerns::NotificationCenter::Notifiable
    
    hermes_tools 'revenues'

    REVENUE_FIELDS = %w[
      year_minus_one
      amount_year_minus_one
      year_minus_two
      amount_year_minus_two
    ]

    ELLIGIBILY_FIELDS = %w[
      mutation
      mobility
      priority_public
      single_parent
      monthly_incomes
      monthly_charges
      birth_date
      work_contract_type
      workplace_insee_code
    ] + REVENUE_FIELDS
    

    FORM_VALIDATOR_FIELD = %w[
      main_employer_name
      work_contract_type
      civility
      last_name
      first_name
      birth_date
      nationality
      email
      postal_details
      workplace_insee_code
    ] + REVENUE_FIELDS
    
    field :test_created_at, type: Date
    
    field :temporary_id, type: String
    
    field :nud, type: String
    field :sne_profile_creation_date, type: DateTime
    field :ss_id, type: String
    
    field :first_name, type: String
    field :last_name, type: String
    field :patronymic_name, type: String
    field :nationality, type: String
    field :sms_notification, type: Boolean
    field :email, type: String
    field :email_advances_assistant, type: String
    field :event_notification, type: Boolean
    
    field :cellphone, type: String
    field :homephone, type: String
    field :workphone, type: String
    
    field :civility, type: String
    
    field :birth_date, type: Date
    
    field :work_contract_type, type: String
    field :profession_details, type: Hash
    field :has_other_employers, type: Boolean
    field :main_employer_name, type: String
    field :main_employer_siret, type: String
    field :workplace_postal_code, type: String
    field :workplace_locality, type: String
    field :foreign_workplace_postal_code, type: String
    field :foreign_workplace_locality, type: String
    field :foreign_workplace_country, type: String
    
    field :year_minus_one, type: String
    field :amount_year_minus_one, type: Float
    field :year_minus_two, type: String
    field :amount_year_minus_two, type: Float
    field :ressources_details, type: Hash
    field :monthly_incomes, type: Hash
    field :monthly_charges, type: Hash
    field :tax_revenues, type: Float
    field :apl_amount, type: Float
    field :family_situation, type: String
    field :family_situation_validated, type: Boolean
    
    field :asked_children_nb, type: Integer
    field :valid_children_nb, type: Integer
    
    field :situation_tax_treshold, type: Integer
    field :asked_locataire_hlm, type: Boolean
    field :valid_locataire_hlm, type: Boolean
    field :valid_priority_public, type: Boolean
    field :asked_priority_public, type: Boolean
    
    field :single_parent, type: Boolean
    field :priority_public, type: Boolean
    field :mobility, type: Boolean
    field :mutation, type: Boolean
    
    field :completed_requests_nb, type: Integer
    field :on_going_request_nb, type: Integer
    
    field :actual_housing_details, type: Hash, default: {}
    field :postal_details, type: Hash, default: {}
    field :is_owner, type: Boolean
    field :owned_housing_details, type: Hash, default: {}
    
    field :disability, type: String
    field :renewal_date, type: DateTime
    
    field :main_user_id, type: BSON::ObjectId
    field :workplace_insee_code, type: String
    field :last_xml_flux_36
    #10095 ajout de nouveaux champs cerfa v4
    field :unemployment_period, type: Boolean
    field :unemployment_period_date_end, type: Date
    field :command_to_leave_date, type: Date
    field :pro_or_home_tel_number, type: String
    
    common_indexes *%i[
      actual_housing_details
      amount_year_minus_one
      amount_year_minus_two
      apl_amount
      birth_date
      cellphone
      civility
      completed_requests_nb
      created_at
      email
      event_notification
      family_situation
      first_name
      has_other_employers
      is_owner
      last_name
      main_user_id
      mobility
      mutation
      nationality
      on_going_request_nb
      patronymic_name
      priority_public
      ressources_details
      situation_tax_treshold
      sne_profile_creation_date
      ss_id
      tax_revenues
      updated_at
      work_contract_type
      year_minus_one
      year_minus_two
    ]
    
    
    # temporary fields to handle user side effects
    field :name_will_change, type: Boolean, default: false
    field :email_will_change, type: Boolean, default: false
    
   # validates :main_employer_siret, length: {is: 14}, allow_blank: true
    has_many(
      :housing_requests,
      class_name: 'Hermes::HousingRequest'
    )
    has_many(
      :notifications,
      class_name: '::NotificationCenter::Notification',
      foreign_key: :targetable_id,
      dependent: :destroy,
      order: 'created_at DESC'
    )

    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id
    writer_auth_roles 'collaborateur entreprise', "collaborateur offre logement", 'superadministrateur'
    
    # validates_presence_of :first_name, :last_name
    
    before_create :before_create_actions
    before_save :before_actions # https://dev.hermes.openstack.local/admin/families/5d1239624ac06e4ca38c1670/mapping
    after_save :after_actions
    
    Applicant.add_status(:single)
    
    before_destroy :before_destroy_actions

    ###############################
    #### Notification #############
    def self.targets_selector(applicant, opt = {})
      [applicant]
    end

    def notification_mailer
      ::Hermes::HermesMailer
    end

    def notifiable_title(key, opt={})
      I18n.t("#{key}.title")
    end

    def notifiable_body(key, opt={})
      I18n.t("#{key}.body")
    end

    def notifiable_path
      "https://als-hermes-int.be-idys.com/housing_applications/#{id}"
    end
    ###############################
    ###############################

    def get_amount_year(year)
      case year.to_s 
      when year_minus_one.to_s
        amount_year_minus_one
      when year_minus_two.to_s
        amount_year_minus_two
      else
        nil
      end
    end 
    
    def dynamic_year_minus_two
      get_minus_two_year.to_s
    end 

    def dynamic_year_minus_one
      (get_minus_two_year.to_i + 1).to_s
    end 

    def dynamic_amount_minus_two
      get_amount_year(dynamic_year_minus_two)
    end 

    def dynamic_amount_minus_one
      get_amount_year(dynamic_year_minus_one)
    end 
    
    def before_destroy_actions
      cleanup_user
      destroy_applicant_link
      clear_coapplicants
      destroy_required_documents
      remove_applicant_used_for_siret_in_request
    end
    
    def remove_applicant_used_for_siret_in_request
      requests = Hermes::HousingRequest.where('$or' => [{applicant_used_for_siret: id.to_s}, {applicant_used_for_siret: id}])
      requests.each do |r|
        r.applicant_used_for_siret = nil
        r.save
      end
    end

    def destroy_applicant_link
      applicant_links = Hermes::ApplicantLink.where('$or' => [{applicant_ids: id.to_s}, {applicant_ids: id}])
      applicant_links.each do |al|
        if al.applicant_ids.length > 1
          
          al.applicant_ids.delete(id.to_s)
          al.applicant_ids.delete(id)
          al.save
        else
          al.destroy
        end
      end
    end

    def clear_coapplicants
      Hermes::HousingRequest.where(applicant_ids: id.to_s).each do |hr|
        hr.remove_coapplicant(id)
        hr.save
      end 
    end 
    
    def destroy_required_documents
      Hermes::RequiredDocument.where(applicant_id: id).destroy_all
    end
    
    def cleanup_user
      if user && (user.main_applicant_id == id)
        errors.add(:base, 'Cannot destroy my applicant')
        false
        # self.user.set(main_applicant_id: nil)
      end
    end
    
    def links(type: nil)
      links = Hermes::ApplicantLink.where(applicant_ids: id.to_s)
      links = links.where(type: type) if type.present?
      Hermes::Applicant.where(:id.in => (links.pluck(:applicant_ids).flatten.compact.uniq - [id.to_s]))
    end
    
    def link_types_for(applicant_id)
      Hermes::ApplicantLink.where(:$and => [{applicant_ids: id.to_s}, {applicant_ids: applicant_id.to_s}]).pluck(:type).uniq
    end

    def last_created_request
      @last_created_request ||= Hermes::HousingRequest.order_by(created_at: :desc).find_by(main_applicant_id: self.id)
    end 
    
    def first_active_request
      @first_active_request ||= Hermes::HousingRequest.find_by(main_applicant_id: self.id, is_active: true)
    end
    
    def coapplicant_first_active_request
      @coapplicant_first_active_request ||= Hermes::HousingRequest.order_by(created_at: :desc).find_by(is_active: true, applicant_ids: self.id.to_s)
    end 

    def request_where_applicant_used_for_siret
      @request_where_applicant_used_for_siret ||= Hermes::HousingRequest.find_by(
        is_active: true, 
        '$or': [
          {applicant_used_for_siret: id},
          {applicant_used_for_siret: nil, main_applicant_id: id},
        ]
      )
    end 

    def coapplicants
      first_active_request&.coapplicants
    end
    
    def coapplicant_ids
      first_active_request&.coapplicants.map {|a| a[:id]}
    end
    
    def monthly_revenues
      (monthly_incomes || {}).values_fsum - (monthly_charges || {}).values_fsum
    end
    
    def get_applicant_attached_codes
      ([actual_housing_details&.fetch('address_insee_code', nil), workplace_insee_code] - [nil, ""]).uniq
    end 

    def form_validatator_field_changed?
      FORM_VALIDATOR_FIELD.any? { |a| send("#{a}_changed?")}
    end 

    def update_linked_request(request)
      request.update_monthly_revenues if monthly_incomes_changed? || monthly_charges_changed?
      request.update_category if family_situation_changed? || birth_date_changed?
      request.update_tax_revenues if REVENUE_FIELDS.any? { |a| send("#{a}_changed?")}
      request.update_eligibility_query if ELLIGIBILY_FIELDS.any? { |a| send("#{a}_changed?")}
      request.update_applicants_attached_codes if actual_housing_details_changed? || workplace_insee_code_changed?
      request.should_update_score = true if work_contract_type_changed?
      request.should_update_entity_relationship = true if work_contract_type_changed?
    end 

    def update_first_active_request
      return unless first_active_request&.reload

      first_active_request.update_requester_data if identity_values_changed? || birth_date_changed?
      update_linked_request(first_active_request)

      first_active_request.save! if first_active_request.changed? || form_validatator_field_changed?
    end 

    def update_coapplicant_first_active_request
      return unless coapplicant_first_active_request&.reload

      update_linked_request(coapplicant_first_active_request)

      coapplicant_first_active_request.save! if coapplicant_first_active_request.changed? || form_validatator_field_changed?
    end 

    def update_request_where_applicant_used_for_siret
      return unless request_where_applicant_used_for_siret&.reload

      request_where_applicant_used_for_siret.update_entity_relationship if main_employer_siret_changed?
      request_where_applicant_used_for_siret.calculate_score if work_contract_type_changed?
      request_where_applicant_used_for_siret.update_applicant_used_for_siret_info if identity_values_changed? || birth_date_changed?

      request_where_applicant_used_for_siret.save! if request_where_applicant_used_for_siret.changed? || form_validatator_field_changed?
    end 

    def main_user
      User.find_by(id: main_user_id)
    end
    
    def identity_values_changed?
      first_name_changed? || last_name_changed? || email_changed?
    end
    
    def needs_first_identity_set?
      new_record? && !identity_values_changed?
    end
    
    def full_name
      [first_name, last_name].join(' ')
    end
    
    def before_create_actions
      self.writers = [user_id.to_s] if user_id
    end 

    def before_actions
      self.main_user_id = user_id if main_user_id.blank?
    end

    def update_housing_request_folders_if_needed
      is_needed = (nationality_changed? ||
        work_contract_type_changed? ||
        monthly_incomes_changed? ||
        monthly_charges_changed? ||
        family_situation_changed? ||
        actual_housing_details_changed? ||
        amount_year_minus_one_changed? ||
        amount_year_minus_two_changed? ||
        year_minus_one_changed? ||
        year_minus_two_changed?
      
      )
      return unless is_needed
      
      hrs = HousingRequest.any_of({main_applicant_id: id}, {main_applicant_id: id.to_s}, {applicant_ids: id}, applicant_ids: id.to_s)
      
      hrs.each do |concerned_hr|
        concerned_hr.housing_request_folder.compute_required_documents
        concerned_hr.housing_request_folder.save
      end
    end

    def update_required_documents_user_id
      required_documents = Hermes::RequiredDocument.where(applicant_id: id)
      required_documents.each do |rd|
        rd.writers.push(user_id)
        rd.save
      end
    end

    def after_actions
      check_status_changed
      update_first_active_request
      update_coapplicant_first_active_request
      update_request_where_applicant_used_for_siret
      update_housing_request_folders_if_needed
      update_required_documents_user_id if user_id_changed?
      true
    end

    def self.find_or_create_by_nud(user, unique_number, birth_date = nil)
      # birth_date format: "2020-04-08"
      nud = Hermes::Nud.find_by(unique_number: unique_number)
      return nud.applicant if nud
      
      hri = Hermes::HousingRequestInitializer.find_or_create_by(unique_number: unique_number) do |new_hri|
        # Enter here only if a new record
        applicant = self.new
        new_hri.save
        new_hri.get_xml_from_sne(user, unique_number)
        last = new_hri.last_xml
        unless last
          applicant.errors.add(:nud, :invalid_unique_number)
          return applicant
        end
        
        parsed_xml = Hash.from_xml(last.gsub("\n", ""))
        main_applicant_data = parsed_xml["interfaceNuu"]["demande"]["demandeLogement"]["personnePhysique"]["demandeur"]
        if main_applicant_data['dateNaissance'] != birth_date
          applicant.errors.add(:birth_date, :invalid_birth_date)
          return applicant
        end
        
        applicant = self.find_or_create_by(email: main_applicant_data["mel"])
        unless applicant.user
          user = ::User.create(
            email: main_applicant_data["mel"],
            first_name: main_applicant_data['prenom'],
            last_name: main_applicant_data['nom'],
            auth_roles: ['demandeur'],
            password: SecureRandom.hex(8)
          )
          applicant.update(user: user)
        end
        new_hri.hydrate_housing_request_from_sne(applicant.user.id)
      end
      
      hri.housing_request.main_applicant
    end
  end
end
