require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

describe Hermes::Api::Dmo::HousingApplicationsController, :type => :api do
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers

  check_mutate_params(
    model: Hermes::HousingApplication, 
    mutate_params: Hermes::Api::Dmo::HousingApplicationsController.mutate_params, 
    factory: :housing_application,
    other_factory:[
      {factory: :active_request, id: 'housing_request1'},
      {factory: :active_request, id: 'housing_request2'},
      {factory: :housing_offer, id: 'offer1'},
      {factory: :housing_offer, id: 'offer2'},
    ],
    min_params: {
      housing_request_id: 'housing_request1',
      housing_offer_id: 'offer1'
    }
  )

  let!(:responsible) {create(:user, roles: ['Administrator'])}
  let!(:superadministrateur) {create(:user, auth_roles: ['superadministrateur'])}
  let!(:user) {create(:user)}
  let!(:user_creator) {create(:user, email: 'johndoe@mail.com')}
  let!(:applicant) {create(:applicant, user: user, first_name: "martin", last_name: "Matin")}

  let!(:housing_offer) {create(:housing_offer, external_ref: "3615")}
  let!(:housing_request) {create(:housing_request, main_applicant_id: applicant.id.to_s, applicant_ids: nil,  is_active: true, user: user_creator)}
  let!(:nud) { create(:nud, housing_request: housing_request, unique_number: '1', provider: 'IMHOWEB')}

  RSpec.shared_examples_for 'no_right_to_change_status' do |action|
    let!(:user) {create(:user)}
    let!(:success_code) { action == 'create' ? 201 : 200}

    before do 
      authenticated_header(request, user)
      housing_application.update(writers: [user.id.to_s]) unless action == 'create'
    end 

    it 'works when admin' do 
      user.update(roles: ['Administrator'])
      subject
      expect(response).to have_http_status(success_code)
    end 

    it 'error when not admin' do 
      subject
      expect(response).to have_http_status(422)
      expect(json["errors"].first["title"]).to match(/no_right_to_change_status/)
    end 

    ["EN ATTENTE", "ANNULEE"].each do |status|
      it "works when status #{status}" do
        params[:data][:attributes][:status] = status
        subject
        expect(response).to have_http_status(success_code)
      end 
    end 

    it 'error when other status' do 
      params[:data][:attributes][:status] = 'TRANSMIS BAILLEUR'
      subject
      expect(response).to have_http_status(422)
      expect(json["errors"].first["title"]).to match(/no_right_to_change_status/)
    end 

    unless action == 'create'
      it 'works when status does not change' do
        params[:data][:attributes][:status] = housing_application.status
        subject
        expect(response).to have_http_status(success_code)
      end 
    end
  end

  describe 'POST /housing_applications' do
    let!(:params) {
      {
        data: {
          attributes: {
            housing_offer_id: housing_offer.id.to_s,
            housing_request_id: housing_request.id.to_s,
            status: 'INSCRIPTION CAL',
          }
        }
      }
    }

    subject { post 'create', params }

    before :each do 
      authenticated_header(request, responsible)
    end 

    it 'should has no housing_applciaiton' do
      expect(housing_request.applications_nb).to be 0
    end

    it 'returns status code 201' do
      subject
      expect(response).to have_http_status(201)
      expect(housing_request.reload.applications_nb).to be 1
    end

    it 'error without skip_check option' do 
      housing_request.set(supplement_can_apply: false)
      post 'create', params.merge(option: ['skip_check'])
      expect(response).to have_http_status(422)
    end 

    it 'with skip_check option' do 
      housing_request.set(supplement_can_apply: false)
      post 'create', params.merge(options: ['skip_check'])
      expect(response).to have_http_status(201)
    end 

    it_behaves_like 'no_right_to_change_status', 'create'
  end

  describe 'GET /housing_applications' do
    let!(:applicant1) {create(:applicant)}
    let!(:applicant2) {create(:applicant)}
    let!(:applicant3) {create(:applicant)}
    let!(:housing_request1) {create(:housing_request, main_applicant_id: applicant1.id.to_s, is_active: true)}
    let!(:housing_request2) {create(:housing_request, main_applicant_id: applicant2.id.to_s, is_active: true)}
    let!(:housing_request3) {create(:housing_request, main_applicant_id: applicant3.id.to_s, is_active: true)}

    let!(:housing_application1) do
      create(
        :housing_application,
        housing_offer_id: housing_offer.id.to_s,
        housing_request_id: housing_request.id.to_s
      )
    end

    let!(:housing_application3) do
      create(
        :housing_application,
        housing_offer_id: housing_offer.id.to_s,
        housing_request_id: housing_request2.id.to_s
      )
    end

    let!(:housing_application2) do
      create(
        :housing_application,
        housing_offer_id: housing_offer.id.to_s,
        housing_request_id: housing_request3.id.to_s
      )
    end

    [
      {user: 'responsible', total_objects: 3},
      {user: 'superadministrateur', total_objects: 3},
      {user: 'user', total_objects: 0},
    ].each do |a|
      describe "as #{a[:user]}" do
        before :each do
          authenticated_header(request, eval(a[:user]))
          get 'index'
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(a[:total_objects])
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(a[:total_objects])
        end
      end

    end

    it 'works with unique_numbers filter' do
      housing_request3.set(unique_numbers: ['test_unique_number_filter'])
      authenticated_header(request, responsible)
      get 'index', unique_numbers: 'test_unique_number_filter'
      expect(response).to have_http_status(200)
      expect(json['data'].length).to eq(1)
    end
  end

  describe 'GET /housing_applications/:id' do
    describe 'verif it work' do
      let!(:housing_application1) {create(:housing_application, housing_offer_id: housing_offer.id.to_s,
                                          housing_request_id: housing_request.id.to_s)}
      it 'returns a valid json' do
        authenticated_header(request, responsible)
        get 'show', {id: housing_application1.id, join_housing_offer: true}

        expect(response).to have_http_status(200)
        expect(json).not_to be_empty
        expect(json['data']['attributes']['housing_offer']).not_to be_empty
      end

      it 'check housing_request attributes' do 
        housing_request.get_applicant_used_for_siret.update(
          first_name: 'John',
          last_name: 'Doe',
          email: 'johndoe@email.com',
          birth_date: Date.new(2000, 1, 1),
        )
        housing_request.update(
          seeked_localisation_list: [{"codesPostaux"=>["67000"], "nom"=>"Strasbourg", "codeDepartement"=>"67"}],
          actual_housing_details: {"postal_code"=>"67300", "district"=>"Schiltigheim" },
        )
        authenticated_header(request, responsible)
        get 'show', {id: housing_application1.id, join_housing_offer: true}

        expect(json['data']['attributes']['requester_firstname']).to eq('John')
        expect(json['data']['attributes']['requester_lastname']).to eq('Doe')
        expect(json['data']['attributes']['requester_email']).to eq('johndoe@email.com')
        expect(json['data']['attributes']['requester_birth_date']).to match(/2000-01-01/)
        expect(json['data']['attributes']['applicant_used_for_siret_firstname']).to eq('John')
        expect(json['data']['attributes']['applicant_used_for_siret_lastname']).to eq('Doe')
        expect(json['data']['attributes']['applicant_used_for_siret_email']).to eq('johndoe@email.com')
        expect(json['data']['attributes']['applicant_used_for_siret_birth_date']).to match(/2000-01-01/)
        expect(json['data']['attributes']['seeked_localisation_list'].first['nom']).to eq('Strasbourg')
        expect(json['data']['attributes']['actual_housing_details']['district']).to eq('Schiltigheim')
      end 

      describe 'motive_label' do 
        before :each do
          authenticated_header(request, responsible)
          patch 'update', {
            id: housing_application1.id.to_s,
            data: {
              attributes: {
                motive_code: "test",
              }
            }
          }
        end 

        it 'without config' do 
          get 'show', {id: housing_application1.id}
          expect(json['data']['attributes']['motive_code']).to eq('test')
          expect(json['data']['attributes']['motive_label']).to eq(nil)
        end 

        it 'not mapped' do 
          HERMES_CONFIG['motive_label'] = { 'Lol' => 'lolo' }
          get 'show', {id: housing_application1.id}
          expect(json['data']['attributes']['motive_code']).to eq('test')
          expect(json['data']['attributes']['motive_label']).to eq(nil)
        end 

        it 'when mapped' do 
          HERMES_CONFIG['motive_label'] = { 'test' => 'Libelle' }
          get 'show', {id: housing_application1.id}
          expect(json['data']['attributes']['motive_code']).to eq('test')
          expect(json['data']['attributes']['motive_label']).to eq('Libelle')
        end 

        it 'with link' do 
          HERMES_CONFIG['motive_label'] = { 'test' => "Demande hors périmètre Action Logement Services, voir CGU. <a href=\"/#/legal-notice\">voir CGU</a>" }
          get 'show', {id: housing_application1.id}
          expect(json['data']['attributes']['motive_code']).to eq('test')
          expect(json['data']['attributes']['motive_label']).to include('<a ')
        end 
      end 

    end
    describe 'verif it work not work with a non authorized user' do
      let!(:housing_application1) {create(:housing_application, housing_offer_id: housing_offer.id.to_s,
                                          housing_request_id: housing_request.id.to_s)}

      before :each do
        authenticated_header(request, user)
        get 'show', {id: housing_application1.id}
      end
      it 'returns status code 404' do
        expect(response).to have_http_status(404)
      end
    end
  end

  describe 'PATCH /housing_applications/:id' do
    let!(:user2) {create(:user)}
    let!(:housing_application) do 
      create(:housing_application, 
        housing_offer_id: housing_offer.id.to_s,
        housing_request_id: housing_request.id.to_s, 
        status: "EN ATTENTE",
      )
    end

    subject { patch 'update', params }

    let!(:params) do 
      {
        id: housing_application.id.to_s,
        data: {
          attributes: {
            status: "DESISTEMENT",
          }
        }
      }
    end 

    it 'returns status code 200' do
      authenticated_header(request, responsible)
      patch 'update', params
      expect(response).to have_http_status(200)
    end

    it_behaves_like 'no_right_to_change_status'
  end

  describe 'POST /housing_applications/update_status' do
    let!(:user2) {create(:user)}
    let!(:nud1) do
      create(
        :nud,
        housing_request: housing_request,
        department: '12',
        creation_date: '2016-11-11T14:53:22',
        unique_number: "A1253EF84F3"
      )
    end
    let!(:housing_application) {create(:housing_application, user_id: user2.id.to_s, housing_offer_id: housing_offer.id.to_s,
                                        housing_request_id: housing_request.id.to_s, status: "RETENUE EN CAL",)}
    let!(:params) do
      {
        data: {
          attributes: {
            status: "REFUSEE EN CAL",
            unique_number: housing_request.unique_numbers.last,
            external_ref: housing_offer.external_ref,
            last_status_update_date: '2018-09-28',
            lessor_event_informations: "en mode confinement chemise-slip",
            date_event: '2018-09-26'
          }
        }
      }
    end

    subject { post 'update_status', params }
    
    before :each do
      authenticated_header(request, responsible)
      enqueued_jobs.clear
    end

    it 'with right status' do
      subject
      expect(response).to have_http_status(200)
      expect(json["data"]["attributes"]["status"]).to eq("REFUSEE EN CAL")
      expect(json["data"]["attributes"]["last_status_update_date"]).to eq('2018-09-28')
      expect(json["data"]["attributes"]["date_event"]).to eq('2018-09-26')
      expect(json["data"]["attributes"]["lessor_event_informations"]).to eq("en mode confinement chemise-slip")
      expect(enqueued_jobs.select{|job| job[:job] == ExecuteSopraTaskJob }.length).to eq(1)
      expect(enqueued_jobs.select{|job| job[:job] == ExecuteSopraTaskJob}.first[:args][1]).to include('DESCAL03')
    end

    it 'signature' do
      params[:data][:attributes][:status] = 'LOGE'
      subject
      expect(response).to have_http_status(200)
      expect(json["data"]["attributes"]["status"]).to eq("LOGE")
      expect( housing_offer.reload.offer_status).to eq("LOGEMENT PLACE")
      expect( housing_offer.reload.lessor_status).to eq("PLACE")
      expect(enqueued_jobs.select{|job| job[:job] == ExecuteSopraTaskJob }.length).to eq(1)
      expect(enqueued_jobs.select{|job| job[:job] == ExecuteSopraTaskJob}.first[:args][1]).to include('RADATT')
    end

    [
      :status,
      :unique_number,
      :external_ref
    ].each do |type|
      it "missing_#{type}" do 
        params[:data][:attributes].delete(type.to_sym)
        subject
        expect(response).to have_http_status(422)
        expect(json["errors"].first["title"]).to match(/missing_#{type}/)
        expect(enqueued_jobs.select{|job| job[:job] == ExecuteSopraTaskJob }.length).to eq(0)
      end 
    end 

    it 'housing_request_not_found' do
      params[:data][:attributes][:unique_number] = 'WRONG'
      subject
      expect(response).to have_http_status(422)
      expect(json["errors"].first["title"]).to match(/housing_request_not_found/)
      expect(enqueued_jobs.select{|job| job[:job] == ExecuteSopraTaskJob }.length).to eq(0)
    end   

    it 'housing_application_not_found' do 
      housing_application.update(status: 'ANNULEE')
      enqueued_jobs.clear
      subject
      expect(response).to have_http_status(422)
      expect(json["errors"].first["title"]).to match(/housing_application_not_found/)
      expect(enqueued_jobs.select{|job| job[:job] == ExecuteSopraTaskJob }.length).to eq(0)
    end 

    it 'external_ref_not_matching_application' do 
      params[:data][:attributes][:external_ref] = 'WRONG'
      subject
      expect(response).to have_http_status(422)
      expect(json["errors"].first["title"]).to match(/external_ref_not_matching_application/)
      expect(enqueued_jobs.select{|job| job[:job] == ExecuteSopraTaskJob }.length).to eq(0)
    end 

    it 'not_writers' do 
      authenticated_header(request, user)
      subject
      expect(response).to have_http_status(422)
      expect(json["errors"].first["title"]).to match(/not_writers/)
      expect(enqueued_jobs.select{|job| job[:job] == ExecuteSopraTaskJob }.length).to eq(0)
    end 

    it 'with not in order status' do
      params[:data][:attributes][:status] = 'INSCRIPTION CAL'
      subject
      expect(response).to have_http_status(200)
      expect(json["data"]["attributes"]["status"]).to eq("RETENUE EN CAL")
      expect(enqueued_jobs.select{|job| job[:job] == ExecuteSopraTaskJob }.length).to eq(1)
      expect(enqueued_jobs.select{|job| job[:job] == ExecuteSopraTaskJob}.first[:args][1]).to include('INSCAL')
    end

    it 'with visit_status' do 
      params[:data][:attributes][:status] = 'VISITE PROPOSEE'
      subject
      expect(response).to have_http_status(200)
      expect(json["data"]["attributes"]["status"]).to eq("RETENUE EN CAL")
      expect(json["data"]["attributes"]["visit_status"]).to eq('VISITE PROPOSEE')
      expect(enqueued_jobs.select{|job| job[:job] == ExecuteSopraTaskJob }.length).to eq(1)
      expect(enqueued_jobs.select{|job| job[:job] == ExecuteSopraTaskJob}.first[:args][1]).to include('VISPRO')
    end 
    
    it_behaves_like 'no_right_to_change_status'
  end

  describe 'DELETE /housing_applications/:id' do
    let!(:housing_application) {create(:housing_application, housing_offer_id: housing_offer.id.to_s,
                                        housing_request_id: housing_request.id.to_s)}
    it 'returns status code 404' do
      authenticated_header(request, user)
      delete 'destroy', {id: housing_application.id.to_s }
      expect(response).to have_http_status(404)
    end

    it 'returns status code 204' do
      authenticated_header(request, superadministrateur)
      delete 'destroy', {id: housing_application.id.to_s }
      expect(response).to have_http_status(204)
    end
    
  end

  describe 'POST /housing_applications/attribution' do
    let!(:lessor) { create(:lessor, user_id: responsible.id, title: 'Lessor') }
    let!(:applicant) { create(:applicant) }
    let!(:housing_offer) { create(:housing_offer, lessor: lessor, requalification: true) }
    let!(:lessor_objective) {create(:lessor_objective, department: 'acceuillant', user_id: responsible.id.to_s, lessor: lessor, housing_offer_ids: [housing_offer.id.to_s])}

    before :each do
      authenticated_header(request, responsible)
      applicant.housing_requests.delete_all
    end

    it 'should create a new application with existing applicant' do
      post :attribution, data: { attributes: { applicant_id: applicant.id, housing_offer_id: housing_offer.id } }
      expect(response.status).to eq(201)
      expect(json['data']['attributes']['applicant_id']).to eq(applicant.id.to_s)
      expect(json['data']['attributes']['housing_offer_id']).to eq(housing_offer.id.to_s)
      expect(json['data']['attributes']['housing_request_id']).not_to be_nil
      expect(applicant.housing_requests.count).to eq(1)
    end

    it 'should create a new application with non existing applicant' do
      expect {
        post :attribution, data: { attributes: { nud: '1110121021518G1104', birth_date: '1982-10-10', housing_offer_id: housing_offer.id } }
      }.to change{ Hermes::HousingApplication.count }.by 1
      expect(response.status).to eq(201)
      expect(json['data']['attributes']['applicant_id']).not_to be_nil
      expect(json['data']['attributes']['housing_offer_id']).to eq(housing_offer.id.to_s)
      expect(json['data']['attributes']['housing_request_id']).not_to be_nil
    end
  end
end

require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::HousingApplicationsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  tags = 'API_DMO_HOUSING_APPLICATIONS'

  
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
  let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}
  let!(:superadministrateur) {create(:user, auth_roles: ['superadministrateur'])}
  let!(:applicant) {create(:applicant, user: responsible)}
  let!(:housing_offer) {create(:housing_offer, external_ref: "CVL-000014")}
  let!(:housing_request) {create(:housing_request, main_applicant_id: applicant.id.to_s, is_active: true, user: user)}
  let!(:admin) {create(:administrator)}

  path '/api/dmo/housing_applications' do
    get 'Retrieves all housing_applications' do
      tags tags
      produces 'application/json'

      Hermes::Api::Dmo::HousingApplicationsController.acceptable_params.sort.each do |params_name, options|
        parameter name: params_name.to_s, in: options[:in] || :query, type: options[:type], required: options[:required] || false, description: options[:description] || ''
      end

      security [Bearer: {}]
      paginator

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }

        run_test!
      end 

      response '200', 'housing_applications found' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        let!(:housing_applications) {create_list(:housing_application, 1, housing_offer_id: housing_offer.id.to_s,
                                                  housing_request_id: housing_request.id.to_s
        )}

        include_context 'with integration test'
        include_context 'valid jsonapi collection' do
          let(:expected_collection) {housing_applications}
        end
      end
    end
  end

  path '/api/dmo/housing_applications' do
    post 'Create housing_application' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]

      json_params = {}
      Hermes::Api::Dmo::HousingApplicationsController.mutate_params.each do |key, options|
        json_params[key] = nice_type(options[:type])
      end
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params(json_params),
      )

      let(:params) do
        jsonapi_attributes(
          {housing_offer_id: housing_offer.id.to_s,
            housing_request_id: housing_request.id.to_s}
        )
      end

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }

        run_test!
      end 

      response '201', 'housing_application created' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'

        after do |_example|
          json = JSON.parse(response.body)

          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'housing_applications'
        end
      end
    end
  end

  path '/api/dmo/housing_applications/{id}' do
    get 'Retrieves a single housing_application by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]
      let(:id) {housing_application.id}
      let(:housing_application) { create(:housing_application, housing_offer_id: housing_offer.id.to_s, housing_request_id: housing_request.id.to_s)}

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }
        run_test!
      end 

      response '200', 'housing_application found' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}


        include_context 'with integration test'
        include_context 'valid jsonapi ressource' do
          let(:expected_ressource) {housing_application}
        end
      end
    end
  end

  path '/api/dmo/housing_applications/{id}' do
    put 'edit housing_application' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]
      json_params = {}
      Hermes::Api::Dmo::HousingApplicationsController.mutate_params.each do |key, options|
        json_params[key] = nice_type(options[:type])
      end
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params(json_params),
      )
      let(:params) do
        jsonapi_attributes(
          {
            status: "RETENUE EN CAL"
          }
        )
      end
      let(:housing_application) { create(:housing_application, housing_offer_id: housing_offer.id.to_s, housing_request_id: housing_request.id.to_s)}
      let(:id) {housing_application.id.to_s}

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        let(:id) { 1 }
        run_test!
      end 

      response '200', 'housing_application updated' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}

        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'

        after do |_example|
          json = JSON.parse(response.body)

          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'housing_applications'
        end
      end
    end
  end

  path '/api/dmo/housing_applications/{id}' do
    delete 'Delete a single housing_application by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]
      let(:id) {create(:housing_application, housing_offer_id: housing_offer.id.to_s, housing_request_id: housing_request.id.to_s).id}

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }
        run_test!
      end 

      response '204', 'housing_application deleted' do
        let(:Authorization) {"Bearer #{token_for(superadministrateur)}"}

        run_test!
      end
    end
  end

  path '/api/dmo/housing_applications/update_status' do
    post 'Update status' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]
      json_params = {}
      json_params[:status] = {type: :string}
      json_params[:unique_number] = {type: :string}
      json_params[:external_ref] = {type: :string}
      json_params[:last_status_update_date] = {type: :string}
      json_params[:lessor_event_informations] = {type: :string}

      parameter name: :params, in: :body, schema: jsonify_params(json_params)

      let!(:housing_application) {create(:housing_application, user_id: user.id.to_s, housing_offer_id: housing_offer.id.to_s, housing_request_id: housing_request.id.to_s, status: "INSCRIPTION CAL",)}
      let!(:nud1) do
        create(
          :nud,
          housing_request: housing_request,
          department: '12',
          creation_date: '2016-11-11T14:53:22',
          unique_number: "A1253EF84F3"
        )
      end

      let(:params) do
        jsonapi_attributes(
          {
            "unique_number":"A1253EF84F3",
            "external_ref":"CVL-000014",
            "status":'RETENUE EN CAL',
            "visit_status": 'VISITE PROPOSEE',
            "last_status_update_date":"01/01/0001",
            "lessor_event_informations":"0"
          }
        )
      end

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }

        run_test!
      end 

      response '422', 'missing params' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}
        let(:params) {  jsonapi_attributes({}) }

        run_test!
      end 

      response '200', 'housing_application updated' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'
        run_test!

        after do |_example|
          json = JSON.parse(response.body)
          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'housing_applications'
        end
      end
    end
  end

  path '/api/dmo/housing_applications/attribution' do
    post 'Attribute selected offer to selected applicant' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      json_params = {}
      json_params[:housing_offer_id] = { type: :string }
      json_params[:applicant_id] = { type: :string }
      json_params[:nud] = { type: :string }
      json_params[:birth_date] = { type: :string }
      parameter name: :params, required: true, in: :body, schema: jsonify_params(json_params)
      security [Bearer: {}]

      response '201', 'housing_application created' do
        let!(:Authorization) { "Bearer #{token_for(admin)}" }
        let!(:housing_offer) { create(:housing_offer, :with_lessor, requalification: true) }
        let!(:housing_offer_id) { housing_offer.id }
        let!(:applicant) { create(:applicant) }
        let!(:applicant_id) { applicant.id }
        let!(:params) do
          jsonapi_attributes({
            housing_offer_id: housing_offer_id,
            applicant_id: applicant_id
          })
        end
        run_test!
      end

      response '201', 'housing_application created' do
        let!(:Authorization) { "Bearer #{token_for(admin)}" }
        let!(:housing_offer) { create(:housing_offer, :with_lessor, requalification: true) }
        let!(:housing_offer_id) { housing_offer.id }
        let!(:nud) { '1110121021518G1104' }
        let!(:birth_date) { '1982-10-10' }
        let!(:params) do
          jsonapi_attributes({
            housing_offer_id: housing_offer_id,
            nud: nud,
            birth_date: birth_date
          })
        end
        run_test!
      end

      response '404', 'nud not found' do
        let!(:Authorization) { "Bearer #{token_for(admin)}" }
        let!(:housing_offer) { create(:housing_offer, :with_lessor, requalification: true) }
        let!(:housing_offer_id) { housing_offer.id }
        let!(:nud) { '000000000000000000' }
        let!(:birth_date) { '1982-10-10' }
        let!(:params) do
          jsonapi_attributes({
            housing_offer_id: housing_offer_id,
            nud: nud,
            birth_date: birth_date
          })
        end
        run_test!
      end

      response '422', 'invalid birth_date' do
        let!(:Authorization) { "Bearer #{token_for(admin)}" }
        let!(:housing_offer) { create(:housing_offer, :with_lessor, requalification: true) }
        let!(:housing_offer_id) { housing_offer.id }
        let!(:nud) { '1110121021518G1104' }
        let!(:birth_date) { '1992-05-10' }
        let!(:params) do
          jsonapi_attributes({
            housing_offer_id: housing_offer_id,
            nud: nud,
            birth_date: birth_date
          })
        end
        run_test!
      end
    end
  end
end