require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')
require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::PicturesController, :type => :api do
  
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers
  
  let!(:responsible) {create(:user, roles: ['Administrator'])}
  
  describe 'POST /pictures' do
    let!(:cat_picture) {Rack::Test::UploadedFile.new(File.join(Rails.root, 'packages', 'hermes', 'spec', 'fixtures', 'files', 'cat1.jpg'), 'image/jpeg') }
    let!(:pdf_as_image) {Rack::Test::UploadedFile.new(File.join(Rails.root, 'packages', 'hermes', 'spec', 'fixtures', 'files', 'pdf_as_image.jpg'), 'image/jpeg') }
    let!(:params) { { file: cat_picture} }

    describe 'create' do
      before :each do
        authenticated_header(request, responsible)
      end 

      subject { post 'create', params }

      it 'works' do
        post 'create', params
        expect(response).to have_http_status(201)
      end

      %w[
        rdsl
        lessor
        collaborateur offer logement
      ].each do |role|
        context "when #{role}" do
          let!(:user) {create(:user, auth_roles: [role])}
          it 'works' do
            authenticated_header(request, user)
            subject
            expect(response).to have_http_status(201)
          end
        end
      end 

      it 'when no file' do 
        post 'create', params.except(:file)
        expect(response).to have_http_status(422)
        expect(response.body).to include('no_file_in_the_request')
      end 

      it 'when unsupported media type' do 
        post 'create', params.merge(file: pdf_as_image)
        expect(response).to have_http_status(415)
        expect(response.body).to include('not_an_image')
      end 
    end
  end

  describe 'DELETE /pictures/{:id}' do
    let!(:user) {create(:user)}
    let!(:picture) {create(:picture, user_id: responsible.id)}
    let!(:id) { picture.id.to_s }
    subject { delete 'destroy', id: id }

    before :each do
      authenticated_header(request, responsible)
    end 

    it 'works' do
      subject
      expect(response).to have_http_status(204)
    end 

    context 'when no picture found' do
      let!(:id) { 'test' }
      before {subject }

      it {expect(response).to have_http_status(404)}
    end

    it 'when no access' do
      authenticated_header(request, user)
      subject
      expect(response).to have_http_status(401)
    end 

    it 'when access with offer' do
      offer = create(:housing_offer, readers: [user.id.to_s], ordered_picture_ids: [picture.id.to_s])
      authenticated_header(request, user)
      subject
      expect(response).to have_http_status(204)
      expect(offer.reload.ordered_picture_ids).to_not include(picture.id.to_s)
    end

  end 
end

describe Hermes::Api::Dmo::PicturesController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  TAG = 'API_DMO_PICTURES'
  
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
  let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}

  # Pictures functionalities
  path '/api/dmo/pictures' do
    post 'create_picture' do
      tags TAG
      produces 'application/json'
      parameter name: 'Content-Type', in: :header, type: :string, required: false, description: "must be 'multipart/form-data'"
      consumes 'multipart/form-data'
      # consumes 'application/json'
      security [Bearer: {}]
      parameter name: :file, in: :formData, type: :file, description: 'picture to upload'
      description 'Send a file with the post action to add a picture'

      response '201', 'picture created' do
        let(:file) do
          Rack::Test::UploadedFile.new([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/'), 'mime/type')
        end
        let(:'Content-Type') {'multipart/form-data'}
        let(:Authorization) {"Bearer #{token_for(responsible)}"}
        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'
      end
    end
  end

  path '/api/dmo/pictures/{id}' do
    delete 'delete_picture' do
      tags TAG
      produces 'application/json'
      consumes 'application/json'
      parameter name: :id, in: :path, type: :string, required: true, description: 'ID of the picture'
      security [Bearer: {}]

      response '204', 'picture deleted' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}
        let!(:id) {create(:picture, user_id: responsible.id).id.to_s}

        run_test!
      end
    end
  end
end